% Example of adaptive noise cancellation

N  = 5000;
t  = [0:N-1]';
s  = sin(2*pi*0.055*t); % Primary signal
x  = randn(N,1);        % Noise signal
P  = fir1(11,0.4);      % Eleventh order lowpass filter
xp = filter(P,1,x);     % Correlated noise data
d  = s + xp;

L  = 12;
W  = zeros(N,L); W(L-1,:) = randn(1,L);
mu = 0.025/L; 
y  = zeros(N,1);
e  = zeros(N,1);
J  = ones(N,1)*(d'*d/N);

for n=L:N
   X      = x(n:-1:n-L+1);
   y(n)   = W(n-1,:)*X;
   e(n)   = d(n)-y(n);
   W(n,:) = W(n-1,:) + mu*X'*e(n);
   J(n)   = e(L:n)'*e(L:n)/(n-L);
end

%----------------------------------------------------------------
 
figure(1)
   subplot(211)
   hp1 = plot(s(L:L+500),'b');axis([0 500 -1.5 1.5])
   hold on
   hp2 = plot(e(L:L+500),'g');
   hold off
   set(hp1,'EraseMode','xor');
   set(hp2,'EraseMode','xor');
   subplot(212)
   hp3 = plot(t(L:L+500),J(L:L+500),'r');
   xlabel('CTRL-C to terminate')
   for n=L+1:N-500
      subplot(211),title(['Istante corrente = ' num2str(n+500)])
      set(hp1,'YData',s(n:n+500));
      set(hp2,'YData',e(n:n+500));
      set(hp3,'XData',t(L:n+500));
      set(hp3,'YData',J(L:n+500));
      drawnow
   end


%----------------------------------------------------------------
 
figure(2)
   plot(t,W)

%----------------------------------------------------------------

figure(3)
bode(tf(P,[1 zeros(1,11)],1))
hold on
bode(tf(W(end,:),[1 zeros(1,11)],1))
hold off

%----------------------------------------------------------------
 
xc = xcorr(x,L);
R  = [];
for iR=1:L
   R = [R xc(L+2-iR:2*L+1-iR)];
end
dxc  = xcorr(d,x,L);
p    = dxc(L+1:2*L);
wopt = R\p;
disp(['max autovalore di R = ' num2str(max(eig(R)))])
disp(['min autovalore di R = ' num2str(min(eig(R)))])
disp('Confronto pesi stimati (1a col.), ottimi (2a col.) e veri (3a col.)')
disp([W(end,:)' wopt P'])