function BBANC_BP_FXLMS(Stype,alg)
% function BBANC_BP_FXLMS(Stype,alg)
% Broadband ANC
% Disturbance = Bandpass filtered WN
% Single long filter

Ts = 0.0001;
N  = 200;

flow  = 100;
fhigh = 200;
[B,A] = butter(2,[flow fhigh]*2*Ts);
bode(tf(B,A,Ts))

L  = 40;
mu = 0.01; 

W  = zeros(L,1);
X  = zeros(L,1);
U  = zeros(length(B),1);
D  = zeros(length(A)-1,1);
J  = 0;

P  = [-1.3 1.6 -1.2 0.17 -0.37 0.3 -0.05 0.75 0.95 2.95];
if Stype==1
   S = [0.9 0.8 0.7 -0.6 -0.5 -0.4 0.3 0.2 0.1 0.5]; % S con poco sfasamento
elseif Stype==2
   S = [0.02 -0.0199];  % S sfasata di 90 circa in banda
else
   disp('Possible values for Stype = 1, 2')
   return
end


Y  = zeros(length(S),1);
XS = zeros(L,1);

Dv = zeros(N,1);
Ev = zeros(N,1);
Jv = NaN;
%Wv = zeros(N,L);

dd = zeros(50*N,1);
ee = zeros(50*N,1);

figure(1)
subplot(211)
hp1a = plot(0:Ts:(N-1)*Ts,Dv,'b');
hold on
hp1b = plot(0:Ts:(N-1)*Ts,Ev,'g');
hold off
set(hp1a,'EraseMode','xor');
set(hp1b,'EraseMode','xor');

t  = 0;
ni = 0;
while 1
   ni = ni + 1;
   t  = t + Ts;
   u  = randn; 
   U  = [u; U(1:end-1)];
   x  = [B -A(2:end)]*[U; X(1:length(A)-1)];  
   X  = [x; X(1:end-1)];
   d  = P*X(1:length(P));  
   D  = [d; D(1:end-1)];

   y  = W'*X;
   Y  = [y; Y(1:end-1)];
   yS = S*Y(1:length(S));  
   xS = S*X(1:length(S));  
   XS = [xS; XS(1:end-1)];
   
   e  = d - yS;
   if alg==1
      W = W + mu*X*e/(X'*X);  % LMS (normalized)
   elseif alg==2
      W = W + mu*XS*e/(XS'*XS); % FXLMS (normalized)
   else
      disp('Possible values for alg = 1 (LMS), 2 (FXLMS)')
      return
   end
   
   J  = (J*(ni-1)+e'*e)/ni;

   Dv = [Dv(2:end);d];
   Ev = [Ev(2:end); e];
%   Wv = [Wv(2:end,:); W'];
   if rem(ni,N)==0
      Jv = [Jv; J];
      figure(1)
      subplot(211),title(['Current time = ' num2str(t)])
      set(hp1a,'XData',t-(N-1)*Ts:Ts:t);
      set(hp1a,'YData',Dv);
      set(hp1b,'XData',t-(N-1)*Ts:Ts:t);
      set(hp1b,'YData',Ev);
      axis([t-(N-1)*Ts t -2 2])
      drawnow
      subplot(212)
      plot([0:length(Jv)-1]*N*Ts,Jv)
      axis([0 t 0 max(Jv)])
      xlabel('CTRL-C to terminate')
      subplot(111)
   end

   if rem(ni,50*N)==0
      figure(2)
      [M1,P1] = bode(tf(P,[1 zeros(1,9)],Ts),logspace(2,4,1000));
      [M2,P2] = bode(tf(W',[1 zeros(1,L-1)],Ts),logspace(2,4,1000));
      subplot(211)
      plot(logspace(2,4,1000)/2/pi,M1(:),'b')
      hold on
      plot(logspace(2,4,1000)/2/pi,M2(:),'g')
      a = axis;
      plot(flow*[1 1],a(3:4),'k:')
      plot(fhigh*[1 1],a(3:4),'k:')
      hold off
      subplot(212)
      plot(logspace(2,4,1000)/2/pi,P1(:),'b')
      hold on
      plot(logspace(2,4,1000)/2/pi,P2(:),'g')
      a = axis;
      plot(flow*[1 1],a(3:4),'k:')
      plot(fhigh*[1 1],a(3:4),'k:')
      hold off
   end

   dd(rem(ni,50*N)+1) = d;
   ee(rem(ni,50*N)+1) = e;
   if rem(ni,50*N)==0
      figure(3)
      subplot(111)
      psd(dd)
      hold on
      psd(ee)
      a = axis;
      plot(flow*[1 1]*2*Ts,a(3:4),'k:')
      plot(fhigh*[1 1]*2*Ts,a(3:4),'k:')
      hold off
   end
end


